
UNIT WIN32;

(* VERSION 1.00.
   DATE DE CREATION : 16-12-2002.
   DERNIERE MISE A JOUR : 19-07-2004.

   APPEL DES FONCTIONS DU KERNEL 32 BITS DEPUIS UN PROGRAMME 16 BITS.

   INTERFACE HYBRIDE 16 BITS/32 BITS AVEC UN CERTAIN NOMBRE DE FONCTIONS :
   - Function WIN32_GETCAPTURE : LongInt;

   - Function WIN32_CREATEFILE (NomFichier : pChar; Mode : LongInt; var hFichier : LongInt) : Boolean;
   - Function WIN32_OPENFILE (NomFichier : pChar; Mode : LongInt; var hFichier : LongInt) : Boolean;
   - Function WIN32_READFILE (hFichier : LongInt; Buffer : pChar; Taille : LongInt) : LongInt;
   - Function WIN32_WRITEFILE (hFichier : LongInt; Buffer : pChar; Taille : LongInt) : LongInt;
   - Function WIN32_SETFILEPOINTER (hFichier : LongInt; Offset : LongInt; Origine : LongInt) : LongInt;
   - Function WIN32_CLOSEFILE (hFichier : LongInt) : Boolean;
   - Function WIN32_GETSHORTPATHNAME (LongPath, ShortPath : pChar; Taille : LongInt) : Boolean;

   - Function WIN32_FINDFIRST (NomFichier : pChar; Attributs : LongInt; var FD : tWIN32FindData) : Boolean;
   - Function WIN32_FINDNEXT (var FD : tWIN32FindData) : Boolean;
   - Function WIN32_FINDCLOSE (var FD : tWIN32FindData) : Boolean;
   - Function WIN32_GETCURRENTDIRECTORY (TailleChemin : LongInt; Chemin : pChar) : Boolean;

   - Function WIN32_GETMODULEHANDLE (NomModule : pChar) : LongInt;
   - Function WIN32_GETMODULEFILENAME (Module : LongInt; Chemin : pChar; TailleChemin : LongInt) : LongInt;

   - Function WIN32_GETOPENFILENAME (var OFN : tWIN32OpenFileName) : Boolean;
   - Function WIN32_GETSAVEFILENAME (var OFN : tWIN32OpenFileName) : Boolean;
   - Function WIN32_CHOOSECOLOR (var CC : tWIN32ChooseColor) : Boolean;
   - Function WIN32_CHOOSEFONT (var CF : tWIN32ChooseFont; var LF : tWIN32LogFont; DC : LongInt) : Boolean;
   - Function WIN32_COMMDLGEXTENDEDERROR : LongInt;
   LA LISTE CI-DESSUS EST LOIN D'EXHAUSTIVE ET IL EST POSSIBLE D'ENRICHIR L'UNITE WIN32 D'UN TAS D'AUTRES ROUTINES 32 BITS.

   PRINCIPES : - LA FONCTION WIN32_INIT DETECTE S'IL S'AGIT D'UN OS 32 BITS.
               - LES FONCTIONS CI-DESSUS SONT APPELEES DEPUIS LE PROGRAMME 16 BITS. POUR UN OS 32 BITS, C'EST LA VERSION
                 32 BITS QUI EST APPELEE, SINON C'EST LA VERSION 16 BITS (VIA L'UNITE WINPROCS).
               - LA PROCEDURE WIN32_EXIT FERME LES LIBRAIRIES EVENTUELLEMENT OUVERTES.

   APPEL 32 BITS : UTILISATION DES "THUNK FUNCTIONS" DU KERNEL 16 BITS :
                   - LoadLibraryEx32W : Function (NomModule : pChar; hFichier, Flags : LongInt) : LongInt = Nil;
                   - FreeLibrary32W : Function (hModule : LongInt) : LongBool = Nil;
                   - GetProcAddress32W : Function (hModule : LongInt; NomRoutine : pChar) : Pointer = Nil;
                   - GetVDMPointer32W : Function (Pointeur1616 : Pointer; ModeTraduction : Word) : Pointer = Nil;
                   - CallProc32W : Function (AdresseProc : Pointer; Masque, NbParams : LongInt) : LongInt = Nil;
                   COMME LA FONCTION CallProc32W NECESSITE UN NOMBRE VARIABLE DE PARAMETRES (DE 0 A 32), ON PASSE PAR UN
                   TABLEAU DE 32 PARAMETRES APPELE CallProcParams. CE TABLEAU EST PASSE A LA FONCTION Appel32 QUI DEPOSE
                   LES PARAMETRES NECESSAIRES SUR LA PILE AVANT D'APPELER CallProcParams. *)

INTERFACE

(* ---------------------------------------------------------------------------------------------------------------------- *)
(*                                             DIRECTIVES DE COMPILATION                                                  *)
(* ---------------------------------------------------------------------------------------------------------------------- *)

(* ---------------------------------------------------------------------------------------------------------------------- *)
(*                                              UNITES ET DLL UTILISEES                                                   *)
(* ---------------------------------------------------------------------------------------------------------------------- *)

Uses STRINGS,    (* GESTION DES CHAINES AZT *)
     WINDOS,     (* ROUTINES MS-DOS *)
     WINTYPES,   (* TYPES DE L'API 16 BITS WINDOWS *)
     WINPROCS,   (* ROUTINES DE L'API 16 BITS WINDOWS *)
     COMMDLG;    (* DIALOGUES COMMUNS WINDOWS 16 BITS *)

(* ---------------------------------------------------------------------------------------------------------------------- *)
(*                                    CONSTANTES NON TYPEES (DIRECTIVES AU COMPILATEUR)                                   *)
(* ---------------------------------------------------------------------------------------------------------------------- *)

Const (* ---------- PARAMETRES DE W16GetVDMPointer32W ---------- *)

      vdm_AddressIsReal = 0;        (* L'ADRESSE PASSEE A LA FONCTION EST UNE ADRESSE MODE REEL *)
      vdm_AddressIsProtected = 1;   (* L'ADRESSE PASSEE A LA FONCTION EST UNE ADRESSE MODE PROTEGE *)

      (* ---------- MODES D'ACCES A UN FICHIER EN CREATION OU EN OUVERTURE ---------- *)

      Generic_Read    = $80000000;
      Generic_Write   = $40000000;
      Generic_Execute = $20000000;
      Generic_All     = $10000000;

      (* ---------- MODES DE PARTAGE DE FICHIER EN CREATION OU EN OUVERTURE ---------- *)

      File_Share_None   = 0;
      File_Share_Read   = 1;
      File_Share_Write  = 2;
      File_Share_Delete = 3;

      (* ---------- OPTIONS DE CREATION/OUVERTURE DE FICHIER ---------- *)

      Create_New        = 1;
      Create_Always     = 2;
      Open_Existing     = 3;
      Open_Always       = 4;
      Truncate_Existing = 5;
      File_Attribute_ReadOnly   = $00000001;
      File_Attribute_Hidden     = $00000002;
      File_Attribute_System     = $00000004;
      File_Attribute_Archive    = $00000020;
      File_Attribute_Normal     = $00000080;
      File_Attribute_Compressed = $00000800;
      File_Flag_Posix_Semantics  = $01000000;
      File_Flag_Backup_Semantics = $02000000;
      File_Flag_Delete_On_Close  = $04000000;
      File_Flag_Sequential_Scan  = $08000000;
      File_Flag_Random_Access    = $10000000;
      File_Flag_No_Buffering     = $20000000;
      File_Flag_Overlapped       = $40000000;
      File_Flag_Write_Through    = $80000000;

      (* ---------- ORIGINE DU DE PLACEMENT DU POINTEUR DE FICHIER ---------- *)

      Offset_Absolute = 0;   (* DEPLACEMENT DEPUIS LE DEBUT DU FICHIER *)
      Offset_Relative = 1;   (* DEPLACEMENT PAR RAPPORT A LA POSITION ACTUELLE *)
      Offset_FromEnd = 2;    (* DEPLACEMENT DEPUIS LA FIN DU FICHIER *)

(* ---------------------------------------------------------------------------------------------------------------------- *)
(*                                                  TYPES DE DONNEES                                                      *)
(* ---------------------------------------------------------------------------------------------------------------------- *)

Type (* ---------- PARAMETRES DE LA FONCTION W16CallProc32W ---------- *)

     rCallProcParam = Record
                        Valeur : LongInt;     (* DWORD OU ADRESSE *)
                        Traduire : Boolean;   (* INDIQUE S'IL S'AGIT D'UNE ADRESSE 16:16 A TRADUIRE EN ADRESSE LINEAIRE *)
                      end;
     tCallProcParams = Array [0..31] of rCallProcParam;

     (* ---------- ATTRIBUTS DE SECURITE CREATION DE FICHIER ---------- *)

     pW32SecurityAttributes = ^tW32SecurityAttributes;
     tW32SecurityAttributes = Record
                                nLength : LongInt;
                                lpSecurityDescriptor : Pointer;
                                bInheritHandle : LongInt;
                              end;

     (* ---------- RESULTAT DE RECHERCHE DE FICHIERS ---------- *)

     pW32FindData = ^tW32FindData;
     tW32FindData = Record
                      dwFileAttributes : LongInt;
                      ftCreationTime : Array [0..1] of LongInt;
                      ftLastAccessTime : Array [0..1] of LongInt;
                      ftLastWriteTime : Array [0..1] of LongInt;
                      nFileSizeHigh : LongInt;
                      nFileSizeLow : LongInt;
                      dwReserved0 : LongInt;
                      dwReserved1 : LongInt;
                      cFileName : Array [0..261] of Char;
                      cAlternateFileName : Array [0..13] of Char;
                    end;

     tWIN32FindData = Record
                        Fill16 : Array [1..21] of Byte;               (* BUFFER DE LA FONCTION 16 BITS *)
                        Fill32 : Array [0..1] of LongInt;             (* BUFFER DE LA FONCTION 32 BITS *)
                        Attributs : LongInt;
                        DateCreation : Array [0..1] of longInt;
                        DateDernierAcces : Array [0..1] of longInt;   (* DATE DU DERNIER ACCES *)
                        DateModification : Array [0..1] of longInt;
                        Taille : Array [0..1] of LongInt;
                        Nom : Array [0..261] of Char;
                        NomAlt : Array [0..13] of Char;
                        Handle : LongInt;
                      end;

     (* ---------- STRUCTURE D'INTERFACE AVEC LES FONCTIONS GetOpenFileName ET GetSaveFileName ---------- *)

     tW32OpenFileName = Record
                          lStructSize : LongInt;
                          hWndOwner : LongInt;
                          hInstance : LongInt;
                          lpstrFilter : pChar;
                          lpstrCustomFilter : pChar;
                          nMaxCustFilter : LongInt;
                          nFilterIndex : LongInt;
                          lpstrFile : pChar;
                          nMaxFile : LongInt;
                          lpstrFileTitle : pChar;
                          nMaxFileTitle : LongInt;
                          lpstrInitialDir : pChar;
                          lpstrTitle : pChar;
                          Flags : LongInt;
                          nFileOffset : Word;
                          nFileExtension : Word;
                          lpstrDefExt : pChar;
                          lCustData : LongInt;
                          lpfnHook : Pointer;
                          lpTemplateName : pChar;
                        end;

     tWIN32OpenFileName = tOpenFileName;

     (* ---------- STRUCTURE D'INTERFACE AVEC LA FONCTION ChooseColor ---------- *)

     tW32ChooseColor = Record
                         lStructSize : LongInt;
                         hWndOwner : LongInt;
                         hInstance : LongInt;
                         rgbResult : LongInt;
                         lpCustColors : pLongInt;
                         Flags : LongInt;
                         lCustData : LongInt;
                         lpfnHook : Pointer;
                         lpTemplateName : pChar;
                       end;

     tWIN32ChooseColor = tChooseColor;

     (* ---------- STRUCTURES D'INTERFACE AVEC LA FONCTION ChooseFont ---------- *)

     pW32LogFont = ^tW32LogFont;
     tW32LogFont = Record
                     lfHeight : LongInt;
                     lfWidth : LongInt;
                     lfEscapement : LongInt;
                     lfOrientation : LongInt;
                     lfWeight : LongInt;
                     lfItalic : Byte;
                     lfUnderline : Byte;
                     lfStrikeOut : Byte;
                     lfCharSet : Byte;
                     lfOutPrecision : Byte;
                     lfClipPrecision : Byte;
                     lfQuality : Byte;
                     lfPitchAndFamily : Byte;
                     lfFaceName : Array [0..lf_FaceSize - 1] of Char;
                   end;

     tWIN32LogFont = tLogFont;

     tW32ChooseFont = Record
                        lStructSize : LongInt;
                        hWndOwner : LongInt;
                        hDC : LongInt;
                        lpLogFont : pW32LogFont;
                        iPointSize : LongInt;
                        Flags : LongInt;
                        rgbColors : tColorRef;
                        lCustData : LongInt;
                        lpfnHook : Pointer;
                        lpTemplateName : pChar;
                        hInstance : LongInt;
                        lpszStyle : pChar;
                        nFontType : Word;
                        Missing_Alignment : Word;
                        nSizeMin : LongInt;
                        nSizeMax : LongInt;
                      end;

     tWIN32ChooseFont = tChooseFont;

(* ---------------------------------------------------------------------------------------------------------------------- *)
(*                                                  CONSTANTES TYPEES                                                     *)
(* ---------------------------------------------------------------------------------------------------------------------- *)

Const (* ---------- DRAPEAU OS 32 BITS ---------- *)

      OS32Bits : Boolean = False;

      (* ---------- HANDLES DES LIBRAIRIES 16 ET 32 BITS ---------- *)

      hInstanceKernel16 : tHandle = 0;
      hInstanceKernel32 : LongInt = 0;
      hInstanceUser32 : LongInt = 0;
      hInstanceComDlg32 : LongInt = 0;

      (* ---------- ADRESSES DES ROUTINES 16 BITS DE GESTION DES LIBRAIRIES 32 BITS ---------- *)

      W16LoadLibraryEx32W : Function (NomModule : pChar; hFichier, Flags : LongInt) : LongInt = Nil;
      W16FreeLibrary32W : Function (hModule : LongInt) : LongBool = Nil;
      W16GetProcAddress32W : Function (hModule : LongInt; NomRoutine : pChar) : Pointer = Nil;
      W16GetVDMPointer32W : Function (Pointeur1616 : Pointer; ModeTraduction : Word) : Pointer = Nil;
      W16CallProc32W : Function (AdresseProc : Pointer; Masque, NbParams : LongInt) : LongInt = Nil;

      (* ---------- ADRESSES DES ROUTINES 32 BITS ---------- *)

      W32GetCapture : Function : LongInt = Nil;

      W32CreateFile : Function (lpFileName : pChar; dwDesiredAccess, dwShareMode : LongInt;
                                lpSecurityAttributes : Pointer; dwCreationDisposition, dwFlagsAndAttributes : LongInt;
                                hTemplateFile : LongInt) : LongInt = Nil;
      W32ReadFile : Function (hFile : LongInt; lpBuffer : pChar; nNumberOfBytesToRead : LongInt;
                              lpNumberOfBytesRead : pLongInt; lpOverlapped : Pointer) : Boolean = Nil;
      W32WriteFile : Function (hFile : LongInt; lpBuffer : pChar; nNumberOfBytesToWrite : LongInt;
                               lpNumberOfBytesWritten : pLongInt; lpOverlapped : Pointer) : Boolean = Nil;
      W32SetFilePointer : Function (hFile : LongInt; lDistanceToMove : LongInt; lpDistanceToMoveHigh : pLongInt;
                                    dwMoveMethod : LongInt) : LongInt = Nil;
      W32CloseHandle : Function (hObject : LongInt) : Boolean = Nil;

      W32GetShortPathName : Function (lpszLongPath, lpszShortPath : pChar; cchBuffer : LongInt) : LongInt = Nil;

      W32FindFirstFile : Function (lpszSearchFile : pChar; var lpFD : tW32FindData) : LongInt = Nil;
      W32FindNextFile : Function (Handle : LongInt) : LongInt = Nil;
      W32FindClose : Function (Handle : LongInt) : LongInt = Nil;

      W32GetCurrentDirectory : Function (dwSize : LongInt; lpstrCurDir : pChar; Drive : LongInt) : LongInt = Nil;

      W32GetModuleHandle : Function (lpstrModule : pChar) : LongInt = Nil;
      W32GetModuleFileName : Function (hModule : LongInt; lpstrPath : pChar; dwPathSize : LongInt) : LongInt = Nil;

      W32GetOpenFileName : Function (var lpOF : tW32OpenFileName) : LongInt = Nil;
      W32GetSaveFileName : Function (var lpOF : tW32OpenFileName) : LongInt = Nil;
      W32ChooseColor : Function (var lpCC : tW32ChooseColor) : LongInt = Nil;
      W32ChooseFont : Function (var lpCF : tW32ChooseFont) : LongInt = Nil;
      W32CommDlgExtendedError : Function : LongInt = Nil;

(* ---------------------------------------------------------------------------------------------------------------------- *)
(*                                                 VARIABLES GLOBALES                                                     *)
(* ---------------------------------------------------------------------------------------------------------------------- *)

Var CallProcParams : tCallProcParams;   (* PARAMETRES DE W16CallProc32W *)
    CallProcParam : LongInt;            (* PARAMETRE A DEPOSER SUR LA PILE *)

(* ---------------------------------------------------------------------------------------------------------------------- *)
(*                                               PROCEDURES ET FONCTIONS                                                  *)
(* ---------------------------------------------------------------------------------------------------------------------- *)

Function WIN32_INIT : Boolean;
   (* INITIALISATIONS *)
Procedure WIN32_EXIT;
   (* LIBERATION DES LIBRAIRIES CHARGEES *)

Function WIN32_GETCAPTURE : LongInt;
   (* DETERMINATION DU HANDLE 32 BITS DE LA FENETRE DESTINATAIRE DES MESSAGES DE LA SOURIS *)
Function HWND32 (Handle16 : HWnd) : LongInt;
   (* CONVERSION DE HANDLE DE FENETRE 16 BITS EN HANDLE 32 BITS *)

Function WIN32_CREATEFILE (NomFichier : pChar; Mode : LongInt; var hFichier : LongInt) : Boolean;
   (* CREATION D'UN FICHIER *)
Function WIN32_OPENFILE (NomFichier : pChar; Mode : LongInt; var hFichier : LongInt) : Boolean;
   (* OUVERTURE DE FICHIER *)
Function WIN32_READFILE (hFichier : LongInt; Buffer : pChar; Taille : LongInt) : LongInt;
   (* LECTURE DE FICHIER *)
Function WIN32_WRITEFILE (hFichier : LongInt; Buffer : pChar; Taille : LongInt) : LongInt;
   (* ECRITURE DANS UN FICHIER *)
Function WIN32_SETFILEPOINTER (hFichier : LongInt; Offset : LongInt; Origine : LongInt) : LongInt;
   (* DEPLACEMENT DU POINTEUR DE FICHIER *)
Function WIN32_CLOSEFILE (hFichier : LongInt) : Boolean;
   (* FERMETURE D'UN FICHIER *)

Function WIN32_GETSHORTPATHNAME (LongPath, ShortPath : pChar; Taille : LongInt) : Boolean;
   (* DETERMINATION DE LA VERSION 8.3 D'UN NOM LONG DE FICHIER *)

Function WIN32_FINDFIRST (NomFichier : pChar; Attributs : LongInt; var FD : tWIN32FindData) : Boolean;
   (* RECHERCHE DE LA 1ERE OCCURENCE D'UN NOM DE FICHIER *)
Function WIN32_FINDNEXT (var FD : tWIN32FindData) : Boolean;
   (* RECHERCHE DE L'OCCURENCE SUIVANTE D'UN NOM DE FICHIER *)
Function WIN32_FINDCLOSE (var FD : tWIN32FindData) : Boolean;
   (* FIN DE LA RECHERCHE DE FICHIERS *)

Function WIN32_GETCURRENTDIRECTORY (TailleChemin : LongInt; Chemin : pChar) : Boolean;
   (* DETERMINATION DU REPERTOIRE COURANT *)

Function WIN32_GETMODULEHANDLE (NomModule : pChar) : LongInt;
   (* RETOURNE LE HANDLE D'UNE TACHE *)
Function WIN32_GETMODULEFILENAME (Module : LongInt; Chemin : pChar; TailleChemin : LongInt) : LongInt;
   (* DETERMINATION DU NOM DE FICHIER COMPLET DU PROGRAMME EN TRAIN DE S'EXECUTER *)

Function WIN32_GETOPENFILENAME (var OFN : tWIN32OpenFileName) : Boolean;
   (* DIALOGUE COMMUN D'OUVERTURE DE FICHIER *)
Function WIN32_GETSAVEFILENAME (var OFN : tWIN32OpenFileName) : Boolean;
   (* DIALOGUE COMMUN D'ENREGISTREMENT DE FICHIER *)
Function WIN32_CHOOSECOLOR (var CC : tWIN32ChooseColor) : Boolean;
   (* DIALOGUE COMMUN DE CHOIX DE COULEUR *)
Function WIN32_CHOOSEFONT (var CF : tWIN32ChooseFont; var LF : tWIN32LogFont; DC : LongInt) : Boolean;
   (* DIALOGUE COMMUN DE CHOIX DE POLICE DE CARACTERES *)
Function WIN32_COMMDLGEXTENDEDERROR : LongInt;
   (* RETOURNE LE CODE ETENDU DE LA DERNIERE ERREUR RELATIVE AUX DIALOGUES COMMUNS WINDOWS *)

IMPLEMENTATION

Function WIN32_INIT : Boolean;
(* INITIALISATIONS *)
Begin
  WIN32_INIT:=True;
  OS32Bits:=False;
  hInstanceKernel16:=LoadLibrary('Kernel');
  if hInstanceKernel16 >= 32
     then
       begin
         @W16LoadLibraryEx32W:=GetProcAddress(hInstanceKernel16,'LoadLibraryEx32W');
         @W16FreeLibrary32W:=GetProcAddress(hInstanceKernel16,'FreeLibrary32W');
         @W16GetProcAddress32W:=GetProcAddress(hInstanceKernel16,'GetProcAddress32W');
         @W16GetVDMPointer32W:=GetProcAddress(hInstanceKernel16,'GetVDMPointer32W');
         @W16CallProc32W:=GetProcAddress(hInstanceKernel16,'CallProc32W');
         if @W16LoadLibraryEx32W <> Nil
            then
              begin
                (* CHARGEMENT DE LA ROUTINE D'APPEL DES FONCTIONS 32 BITS - DETERMINATION SI C'EST UN OS 32 BITS *)
                hInstanceKernel32:=W16LoadLibraryEx32W('Kernel32',0,0);
                OS32Bits:=(hInstanceKernel32 <> 0);
              end;
       end
     else
       WIN32_INIT:=False;
End;

Procedure WIN32_EXIT;
(* LIBERATION DES LIBRAIRIES CHARGEES *)
Begin
  if @W16FreeLibrary32W <> Nil
     then
       begin
         if hInstanceKernel32 <> 0
            then
              W16FreeLibrary32W(hInstanceKernel32);
         if hInstanceUser32 <> 0
            then
              W16FreeLibrary32W(hInstanceUser32);
         if hInstanceComDlg32 <> 0
            then
              W16FreeLibrary32W(hInstanceComDlg32);
       end;
  if hInstanceKernel16 <> 0
     then
       FreeLibrary(hInstanceKernel16);
  hInstanceKernel16:=0;
  hInstanceKernel32:=0;
  hInstanceUser32:=0;
  hInstanceComDlg32:=0;
End;

Function APPEL32 (NbParams : LongInt; AdresseProc : Pointer) : LongInt;
(* APPEL D'UNE FONCTION WINDOWS 32 BITS *)
Var i : LongInt;        (* INDICE *)
    Masque : LongInt;   (* MASQUE DE TRADUCTION D'ADRESSES 16:16 EN ADRESSES LINEAIRES *)
Begin
  Masque:=$00000000;
  for i:=0 to (NbParams - 1) do
    begin
      if CallProcParams[i].Traduire
         then
           Masque:=Masque or ($00000001 shl i);
      CallProcParam:=CallProcParams[i].Valeur;
      asm
        push word ptr [CallProcParam + 2]
        push word ptr [CallProcParam]
      end;
    end;
  APPEL32:=W16CallProc32W(AdresseProc,Masque,NbParams);
End;

Function WIN32_GETCAPTURE : LongInt;
(* DETERMINATION DU HANDLE 32 BITS DE LA FENETRE DESTINATAIRE DES MESSAGES DE LA SOURIS *)
Begin
  if OS32Bits
     then
       begin
         if @W32GetCapture = Nil
            then
              begin
                if hInstanceUser32 = 0
                   then
                     hInstanceUser32:=W16LoadLibraryEx32W('User32',0,0);
                if hInstanceUser32 <> 0
                   then
                     @W32GetCapture:=W16GetProcAddress32W(hInstanceUser32,'GetCapture');
              end;
         if @W32GetCapture <> Nil
            then
              begin
                FillChar(CallProcParams,SizeOf(CallProcParams),0);
                WIN32_GETCAPTURE:=APPEL32(0,@W32GetCapture);
              end
            else
              WIN32_GETCAPTURE:=0;
       end
     else
       WIN32_GETCAPTURE:=GetCapture;
End;

Function HWND32 (Handle16 : HWnd) : LongInt;
(* CONVERSION DE HANDLE DE FENETRE 16 BITS EN HANDLE 32 BITS *)
Var AncienHandle16 : HWnd;   (* ANCIEN HANDLE 16 BITS DE LA FENETRE *)
Begin
  if @W32GetCapture = Nil
     then
       begin
         if hInstanceUser32 = 0
            then
              hInstanceUser32:=W16LoadLibraryEx32W('User32',0,0);
         if hInstanceUser32 <> 0
            then
              @W32GetCapture:=W16GetProcAddress32W(hInstanceUser32,'GetCapture');
       end;
  if @W32GetCapture <> Nil
     then
       begin
         AncienHandle16:=SetCapture(Handle16);
         HWND32:=WIN32_GETCAPTURE;
         if AncienHandle16 <> 0
            then
              SetCapture(AncienHandle16)
            else
              ReleaseCapture;
       end
     else
       HWND32:=0;
End;

Function WIN32_CREATEFILE (NomFichier : pChar; Mode : LongInt; var hFichier : LongInt) : Boolean;
(* CREATION D'UN FICHIER *)
Var SecurityAttributes : tW32SecurityAttributes;   (* ATTRIBUTS DE SECURITE *)
Begin
  if OS32Bits
     then
       begin
         if @W32CreateFile = Nil
            then
              begin
                if hInstanceKernel32 = 0
                   then
                     hInstanceKernel32:=W16LoadLibraryEx32W('Kernel32',0,0);
                if hInstanceKernel32 <> 0
                   then
                     @W32CreateFile:=W16GetProcAddress32W(hInstanceKernel32,'CreateFileA');
              end;
         if @W32CreateFile <> Nil
            then
              begin
                FillChar(CallProcParams,SizeOf(CallProcParams),0);
                (* lpFileName *)
                CallProcParams[0].Valeur:=LongInt(W16GetVDMPointer32W(NomFichier,StrLen(NomFichier)));
                CallProcParams[0].Traduire:=False;
                (* dwDesiredAccess *)
                CallProcParams[1].Valeur:=Generic_Write;
                CallProcParams[1].Traduire:=False;
                (* dwShareMode *)
                CallProcParams[2].Valeur:=0;
                CallProcParams[2].Traduire:=False;
                (* lpSecurityAttributes *)
{
                FillChar(SecurityAttributes,SizeOf(SecurityAttributes),0);
                SecurityAttributes.nLength:=SizeOf(SecurityAttributes);
                SecurityAttributes.bInheritHandle:=LongInt(True);
                CallProcParams[3].Valeur:=LongInt(@SecurityAttributes);
                CallProcParams[3].Traduire:=True;
}
                CallProcParams[3].Valeur:=0;
                CallProcParams[3].Traduire:=False;
                (* dwCreationDisposition *)
                CallProcParams[4].Valeur:=Create_Always;
                CallProcParams[4].Traduire:=False;
                (* dwFlagsAndAttributes *)
                CallProcParams[5].Valeur:=File_Attribute_Normal;
                CallProcParams[5].Traduire:=False;
                (* hTemplateFile *)
                CallProcParams[6].Valeur:=0;
                CallProcParams[6].Traduire:=False;
                (* APPEL *)
                hFichier:=APPEL32(7,@W32CreateFile);
              end
            else
              hFichier:=-1;
       end
     else
       hFichier:=_lCreat(NomFichier,Mode);
  WIN32_CREATEFILE:=(hFichier <> -1);
End;

Function WIN32_OPENFILE (NomFichier : pChar; Mode : LongInt; var hFichier : LongInt) : Boolean;
(* OUVERTURE DE FICHIER *)
Var n : LongInt;
Begin
  if OS32Bits
     then
       begin
         if @W32CreateFile = Nil
            then
              begin
                if hInstanceKernel32 = 0
                   then
                     hInstanceKernel32:=W16LoadLibraryEx32W('Kernel32',0,0);
                if hInstanceKernel32 <> 0
                   then
                     @W32CreateFile:=W16GetProcAddress32W(hInstanceKernel32,'CreateFileA');
              end;
         if @W32CreateFile <> Nil
            then
              begin
                FillChar(CallProcParams,SizeOf(CallProcParams),0);
                (* lpFileName *)
                CallProcParams[0].Valeur:=LongInt(W16GetVDMPointer32W(NomFichier,StrLen(NomFichier)));
                CallProcParams[0].Traduire:=False;
                (* dwDesiredAccess *)
                CallProcParams[1].Valeur:=Generic_Read;
                CallProcParams[1].Traduire:=False;
                (* dwShareMode *)
                CallProcParams[2].Valeur:=File_Share_Read;
                CallProcParams[2].Traduire:=False;
                (* lpSecurityAttributes *)
                CallProcParams[3].Valeur:=0;
                CallProcParams[3].Traduire:=False;
                (* dwCreationDisposition *)
                CallProcParams[4].Valeur:=Open_Existing;
                CallProcParams[4].Traduire:=False;
                (* dwFlagsAndAttributes *)
                CallProcParams[5].Valeur:=File_Attribute_Normal;
                CallProcParams[5].Traduire:=False;
                (* hTemplateFile *)
                CallProcParams[6].Valeur:=0;
                CallProcParams[6].Traduire:=False;
                (* APPEL *)
                hFichier:=APPEL32(7,@W32CreateFile);
              end
            else
              hFichier:=-1;
       end
     else
       hFichier:=_lOpen(NomFichier,Mode);
  WIN32_OPENFILE:=(hFichier <> -1);
End;

Function WIN32_READFILE (hFichier : LongInt; Buffer : pChar; Taille : LongInt) : LongInt;
(* LECTURE DE FICHIER *)
Var NbBytesLus : LongInt;   (* NOMBRE DE BYTES LUS PAR LA FONCTION 32 BITS *)
Begin
  if OS32Bits
     then
       begin
         if @W32ReadFile = Nil
            then
              begin
                if hInstanceKernel32 = 0
                   then
                     hInstanceKernel32:=W16LoadLibraryEx32W('Kernel32',0,0);
                if hInstanceKernel32 <> 0
                   then
                     @W32ReadFile:=W16GetProcAddress32W(hInstanceKernel32,'ReadFile');
              end;
         if @W32ReadFile <> Nil
            then
              begin
                FillChar(CallProcParams,SizeOf(CallProcParams),0);
                (* hFile *)
                CallProcParams[0].Valeur:=hFichier;
                CallProcParams[0].Traduire:=False;
                (* lpBuffer *)
                CallProcParams[1].Valeur:=LongInt(W16GetVDMPointer32W(Buffer,Taille));
                CallProcParams[1].Traduire:=False;
                (* nNumberOfBytesToRead *)
                CallProcParams[2].Valeur:=Taille;
                CallProcParams[2].Traduire:=False;
                (* lpNumberOfBytesRead *)
                NbBytesLus:=0;
                CallProcParams[3].Valeur:=LongInt(W16GetVDMPointer32W(@NbBytesLus,SizeOf(NbBytesLus)));
                CallProcParams[3].Traduire:=False;
                (* lpOverlapped *)
                CallProcParams[4].Valeur:=0;
                CallProcParams[4].Traduire:=True;
                (* APPEL *)
                if Boolean(APPEL32(5,@W32ReadFile))
                   then
                     WIN32_READFILE:=NbBytesLus
                   else
                     WIN32_READFILE:=0;
              end
            else
              WIN32_READFILE:=0;
       end
     else
       WIN32_READFILE:=_lRead(hFichier,Buffer,Taille);
End;

Function WIN32_WRITEFILE (hFichier : LongInt; Buffer : pChar; Taille : LongInt) : LongInt;
(* ECRITURE DANS UN FICHIER *)
Var NbBytesEcrits : LongInt;   (* NOMBRE DE BYTES ECRITS PAR LA FONCTION 32 BITS *)
Begin
  if OS32Bits
     then
       begin
         if @W32WriteFile = Nil
            then
              begin
                if hInstanceKernel32 = 0
                   then
                     hInstanceKernel32:=W16LoadLibraryEx32W('Kernel32',0,0);
                if hInstanceKernel32 <> 0
                   then
                     @W32WriteFile:=W16GetProcAddress32W(hInstanceKernel32,'WriteFile');
              end;
         if @W32WriteFile <> Nil
            then
              begin
                FillChar(CallProcParams,SizeOf(CallProcParams),0);
                (* hFile *)
                CallProcParams[0].Valeur:=hFichier;
                CallProcParams[0].Traduire:=False;
                (* lpBuffer *)
                CallProcParams[1].Valeur:=LongInt(W16GetVDMPointer32W(Buffer,Taille));
                CallProcParams[1].Traduire:=False;
                (* nNumberOfBytesToWrite *)
                CallProcParams[2].Valeur:=Taille;
                CallProcParams[2].Traduire:=False;
                (* lpNumberOfBytesWritten *)
                NbBytesEcrits:=0;
                CallProcParams[3].Valeur:=LongInt(W16GetVDMPointer32W(@NbBytesEcrits,SizeOf(NbBytesEcrits)));
                CallProcParams[3].Traduire:=False;
                (* lpOverlapped *)
                CallProcParams[4].Valeur:=0;
                CallProcParams[4].Traduire:=False;
                (* APPEL *)
                if Boolean(APPEL32(5,@W32WriteFile))
                   then
                     WIN32_WRITEFILE:=NbBytesEcrits
                   else
                     WIN32_WRITEFILE:=0;
              end
            else
              WIN32_WRITEFILE:=0;
       end
     else
       WIN32_WRITEFILE:=_lWrite(hFichier,Buffer,Taille);
End;

Function WIN32_SETFILEPOINTER (hFichier : LongInt; Offset : LongInt; Origine : LongInt) : LongInt;
(* DEPLACEMENT DU POINTEUR DE FICHIER *)
Begin
  if OS32Bits
     then
       begin
         if @W32SetFilePointer = Nil
            then
              begin
                if hInstanceKernel32 = 0
                   then
                     hInstanceKernel32:=W16LoadLibraryEx32W('Kernel32',0,0);
                if hInstanceKernel32 <> 0
                   then
                     @W32SetFilePointer:=W16GetProcAddress32W(hInstanceKernel32,'SetFilePointer');
              end;
         if @W32SetFilePointer <> Nil
            then
              begin
                FillChar(CallProcParams,SizeOf(CallProcParams),0);
                (* hFile *)
                CallProcParams[0].Valeur:=hFichier;
                CallProcParams[0].Traduire:=False;
                (* lDistanceToMove *)
                CallProcParams[1].Valeur:=Offset;
                CallProcParams[1].Traduire:=False;
                (* lpDistanceToMoveHigh *)
                CallProcParams[2].Valeur:=0;
                CallProcParams[2].Traduire:=False;
                (* DWMoveMethod *)
                CallProcParams[3].Valeur:=Origine;
                CallProcParams[3].Traduire:=False;
                (* APPEL *)
                WIN32_SETFILEPOINTER:=APPEL32(4,@W32SetFilePointer);
              end
            else
              WIN32_SETFILEPOINTER:=0;
       end
     else
       WIN32_SETFILEPOINTER:=_llSeek(hFichier,Offset,Origine);
End;

Function WIN32_CLOSEFILE (hFichier : LongInt) : Boolean;
(* FERMETURE D'UN FICHIER (UTILISATION DES FONCTIONS DU KERNEL WINDOWS) *)
Begin
  if OS32Bits
     then
       begin
         if @W32CloseHandle = Nil
            then
              begin
                if hInstanceKernel32 = 0
                   then
                     hInstanceKernel32:=W16LoadLibraryEx32W('Kernel32',0,0);
                if hInstanceKernel32 <> 0
                   then
                     @W32CloseHandle:=W16GetProcAddress32W(hInstanceKernel32,'CloseHandle');
              end;
         if @W32CloseHandle <> Nil
            then
              begin
                FillChar(CallProcParams,SizeOf(CallProcParams),0);
                (* hObject *)
                CallProcParams[0].Valeur:=hFichier;
                CallProcParams[0].Traduire:=False;
                (* APPEL *)
                WIN32_CLOSEFILE:=(APPEL32(1,@W32CloseHandle) <> -1);
              end
            else
              WIN32_CLOSEFILE:=False;
       end
     else
       WIN32_CLOSEFILE:=(_lClose(hFichier) <> -1);
End;

Function WIN32_GETSHORTPATHNAME (LongPath, ShortPath : pChar; Taille : LongInt) : Boolean;
(* DETERMINATION DE LA VERSION 8.3 D'UN NOM LONG DE FICHIER *)
Begin
  if OS32Bits
     then
       begin
         if @W32GetShortPathName = Nil
            then
              begin
                if hInstanceKernel32 = 0
                   then
                     hInstanceKernel32:=W16LoadLibraryEx32W('Kernel32',0,0);
                if hInstanceKernel32 <> 0
                   then
                     @W32GetShortPathName:=W16GetProcAddress32W(hInstanceKernel32,'GetShortPathNameA');
              end;
         if @W32GetShortPathName <> Nil
            then
              begin
                FillChar(CallProcParams,SizeOf(CallProcParams),0);
                (* lpszLongPath *)
                CallProcParams[0].Valeur:=LongInt(W16GetVDMPointer32W(LongPath,StrLen(LongPath)));
                CallProcParams[0].Traduire:=False;
                (* lpszShortPath *)
                CallProcParams[1].Valeur:=LongInt(ShortPath);
                CallProcParams[1].Traduire:=True;
{
                CallProcParams[1].Valeur:=LongInt(W16GetVDMPointer32W(ShortPath,Taille));
                CallProcParams[1].Traduire:=False;
}
                (* cchBuffer *)
                CallProcParams[2].Valeur:=Taille;
                CallProcParams[2].Traduire:=False;
                (* APPEL *)
                WIN32_GETSHORTPATHNAME:=(APPEL32(3,@W32GetShortPathName) <> 0);
              end
       end
     else   (* FONCTION NON SUPPORTEE PAR LES OS 16 BITS *)
       WIN32_GETSHORTPATHNAME:=False;
End;

Function WIN32_FINDFIRST (NomFichier : pChar; Attributs : LongInt; var FD : tWIN32FindData) : Boolean;
(* RECHERCHE DE LA 1ERE OCCURENCE D'UN NOM DE FICHIER *)
Var FD16 : tSearchRec;     (* STRUCTURE DE RECHERCHE 16 BITS *)
    FD32 : tW32FindData;   (* STRUCTURE DE RECHERCHE 32 BITS *)
Begin
  FillChar(FD,SizeOf(FD),0);
  if OS32Bits
     then
       begin
         if @W32FindFirstFile = Nil
            then
              begin
                if hInstanceKernel32 = 0
                   then
                     hInstanceKernel32:=W16LoadLibraryEx32W('Kernel32',0,0);
                if hInstanceKernel32 <> 0
                   then
                     @W32FindFirstFile:=W16GetProcAddress32W(hInstanceKernel32,'FindFirstFileA');
              end;
         if @W32FindFirstFile <> Nil
            then
              begin
                FD32.dwFileAttributes:=Attributs;
                FillChar(CallProcParams,SizeOf(CallProcParams),0);
                CallProcParams[0].Valeur:=LongInt(NomFichier);
                CallProcParams[0].Traduire:=True;
                CallProcParams[1].Valeur:=LongInt(@FD32);
                CallProcParams[1].Traduire:=True;
                FD.Handle:=APPEL32(2,@W32FindFirstFile);
                if FD.Handle <> -1
                   then
                     begin
                       FD.Fill32[0]:=FD32.dwReserved0;
                       FD.Fill32[1]:=FD32.dwReserved1;
                       FD.Attributs:=FD32.dwFileAttributes;
                       Move(FD32.ftCreationTime,FD.DateCreation,SizeOf(FD.DateCreation));
                       Move(FD32.ftLastAccessTime,FD.DateDernierAcces,SizeOf(FD.DateDernierAcces));
                       Move(FD32.ftLastWriteTime,FD.DateModification,SizeOf(FD.DateModification));
                       FD.Taille[0]:=FD32.nFileSizeLow;
                       FD.Taille[1]:=FD32.nFileSizeHigh;
                       StrCopy(FD.Nom,FD32.cFileName);
                       StrCopy(FD.NomAlt,FD32.cAlternateFileName);
                       WIN32_FINDFIRST:=True;
                     end
                   else
                     WIN32_FINDFIRST:=False;
              end
            else
              WIN32_FINDFIRST:=False;
       end
     else
       begin
         FindFirst(NomFichier,Attributs,FD16);
         if DosError = 0
            then
              begin
                Move(FD16.Fill,FD.Fill16,SizeOf(FD.Fill16));
                FD.Attributs:=FD16.Attr;
                FD.DateModification[0]:=FD16.Time;
                FD.Taille[0]:=FD16.Size;
                StrCopy(FD.Nom,FD16.Name);
                WIN32_FINDFIRST:=True;
              end
            else
              WIN32_FINDFIRST:=False;
       end;
End;

Function WIN32_FINDNEXT (var FD : tWIN32FindData) : Boolean;
(* RECHERCHE DE L'OCCURENCE SUIVANTE D'UN NOM DE FICHIER *)
Var FD16 : tSearchRec;     (* STRUCTURE DE RECHERCHE 16 BITS *)
    FD32 : tW32FindData;   (* STRUCTURE DE RECHERCHE 32 BITS *)
Begin
  if OS32Bits
     then
       begin
         if @W32FindNextFile = Nil
            then
              begin
                if hInstanceKernel32 = 0
                   then
                     hInstanceKernel32:=W16LoadLibraryEx32W('Kernel32',0,0);
                if hInstanceKernel32 <> 0
                   then
                     @W32FindNextFile:=W16GetProcAddress32W(hInstanceKernel32,'FindNextFileA');
              end;
         if @W32FindNextFile <> Nil
            then
              begin
                FillChar(CallProcParams,SizeOf(CallProcParams),0);
                CallProcParams[0].Valeur:=LongInt(FD.Handle);
                CallProcParams[0].Traduire:=False;
                CallProcParams[1].Valeur:=LongInt(@FD32);
                CallProcParams[1].Traduire:=True;
                if Boolean(APPEL32(3,@W32FindNextFile)) = True
                   then
                     begin
                       FD.Fill32[0]:=FD32.dwReserved0;
                       FD.Fill32[1]:=FD32.dwReserved1;
                       FD.Attributs:=FD32.dwFileAttributes;
                       Move(FD32.ftCreationTime,FD.DateCreation,SizeOf(FD.DateCreation));
                       Move(FD32.ftLastAccessTime,FD.DateDernierAcces,SizeOf(FD.DateDernierAcces));
                       Move(FD32.ftLastWriteTime,FD.DateModification,SizeOf(FD.DateModification));
                       FD.Taille[0]:=FD32.nFileSizeLow;
                       FD.Taille[1]:=FD32.nFileSizeHigh;
                       StrCopy(FD.Nom,FD32.cFileName);
                       StrCopy(FD.NomAlt,FD32.cAlternateFileName);
                       WIN32_FINDNEXT:=True;
                     end
                   else
                     WIN32_FINDNEXT:=False;
              end
            else
              WIN32_FINDNEXT:=False;
       end
     else
       begin
         Move(FD.Fill16,FD16.Fill,SizeOf(FD16.Fill));
         FD16.Attr:=FD.Attributs;
         FD16.Time:=FD.DateModification[0];
         FD16.Size:=FD.Taille[0];
         StrCopy(FD16.Name,FD.Nom);
         FindNext(FD16);
         if DosError = 0
            then
              begin
                Move(FD16.Fill,FD.Fill16,SizeOf(FD.Fill16));
                FD.Attributs:=FD16.Attr;
                FD.DateModification[0]:=FD16.Time;
                FD.Taille[0]:=FD16.Size;
                StrCopy(FD.Nom,FD16.Name);
                WIN32_FINDNEXT:=True;
              end
            else
              WIN32_FINDNEXT:=False;
       end;
End;

Function WIN32_FINDCLOSE (var FD : tWIN32FindData) : Boolean;
(* FIN DE LA RECHERCHE DE FICHIERS *)
Begin
  if OS32Bits
     then
       begin
         if @W32FindClose = Nil
            then
              begin
                if hInstanceKernel32 = 0
                   then
                     hInstanceKernel32:=W16LoadLibraryEx32W('Kernel32',0,0);
                if hInstanceKernel32 <> 0
                   then
                     @W32FindClose:=W16GetProcAddress32W(hInstanceKernel32,'FindClose');
              end;
         if @W32FindClose <> Nil
            then
              begin
                FillChar(CallProcParams,SizeOf(CallProcParams),0);
                CallProcParams[0].Valeur:=LongInt(FD.Handle);
                CallProcParams[0].Traduire:=False;
                WIN32_FINDCLOSE:=LongBool(APPEL32(1,@W32FindClose));
              end
            else
              WIN32_FINDCLOSE:=False;
       end;
End;

Function WIN32_GETCURRENTDIRECTORY (TailleChemin : LongInt; Chemin : pChar) : Boolean;
(* DETERMINATION DU REPERTOIRE COURANT *)
Begin
  if OS32Bits
     then
       begin
         if @W32GetCurrentDirectory = Nil
            then
              begin
                if hInstanceKernel32 = 0
                   then
                     hInstanceKernel32:=W16LoadLibraryEx32W('Kernel32',0,0);
                if hInstanceKernel32 <> 0
                   then
                     @W32GetCurrentDirectory:=W16GetProcAddress32W(hInstanceKernel32,'GetCurrentDirectoryA');
              end;
         if @W32GetCurrentDirectory <> Nil
            then
              begin
                FillChar(CallProcParams,SizeOf(CallProcParams),0);
                CallProcParams[0].Valeur:=TailleChemin;
                CallProcParams[0].Traduire:=False;
                CallProcParams[1].Valeur:=LongInt(Chemin);
                CallProcParams[1].Traduire:=True;
                CallProcParams[2].Valeur:=0;
                CallProcParams[2].Traduire:=False;
                WIN32_GETCURRENTDIRECTORY:=(APPEL32(3,@W32GetCurrentDirectory) > 0);
              end
            else
              WIN32_GETCURRENTDIRECTORY:=False;
       end
     else
       begin
         GetCurDir(Chemin,0);
         WIN32_GETCURRENTDIRECTORY:=(DosError = 0);
       end;
End;

Function WIN32_GETMODULEHANDLE (NomModule : pChar) : LongInt;
(* RETOURNE LE HANDLE D'UNE TACHE *)
Begin
  if OS32Bits
     then
       begin
         if @W32GetModuleHandle = Nil
            then
              begin
                if hInstanceKernel32 = 0
                   then
                     hInstanceKernel32:=W16LoadLibraryEx32W('Kernel32',0,0);
                if hInstanceKernel32 <> 0
                   then
                     @W32GetModuleHandle:=W16GetProcAddress32W(hInstanceKernel32,'GetModuleHandleA');
              end;
         if @W32GetModuleHandle <> Nil
            then
              begin
                FillChar(CallProcParams,SizeOf(CallProcParams),0);
                CallProcParams[0].Valeur:=LongInt(NomModule);
                CallProcParams[0].Traduire:=True;
                WIN32_GETMODULEHANDLE:=APPEL32(1,@W32GetModuleHandle);
              end
            else
              WIN32_GETMODULEHANDLE:=0;
       end
     else
       WIN32_GETMODULEHANDLE:=GetModuleHandle(NomModule);
End;

Function WIN32_GETMODULEFILENAME (Module : LongInt; Chemin : pChar; TailleChemin : LongInt) : LongInt;
(* DETERMINATION DU NOM DE FICHIER COMPLET DU PROGRAMME EN TRAIN DE S'EXECUTER *)
Begin
  if OS32Bits
     then
       begin
         if @W32GetModuleFileName = Nil
            then
              begin
                if hInstanceKernel32 = 0
                   then
                     hInstanceKernel32:=W16LoadLibraryEx32W('Kernel32',0,0);
                if hInstanceKernel32 <> 0
                   then
                     @W32GetModuleFileName:=W16GetProcAddress32W(hInstanceKernel32,'GetModuleFileNameA');
              end;
         if @W32GetModuleFileName <> Nil
            then
              begin
                FillChar(CallProcParams,SizeOf(CallProcParams),0);
                CallProcParams[0].Valeur:=Module;
                CallProcParams[0].Traduire:=False;
                CallProcParams[1].Valeur:=LongInt(Chemin);
                CallProcParams[1].Traduire:=True;
                CallProcParams[2].Valeur:=LongInt(TailleChemin);
                CallProcParams[2].Traduire:=False;
                WIN32_GETMODULEFILENAME:=APPEL32(3,@W32GetModuleFileName);
              end
            else
              WIN32_GETMODULEFILENAME:=0;
       end
     else
       WIN32_GETMODULEFILENAME:=GetModuleFileName(Module,Chemin,TailleChemin);
End;

Function WIN32_GETOPENFILENAME (var OFN : tWIN32OpenFileName) : Boolean;
(* DIALOGUE COMMUN D'OUVERTURE DE FICHIER *)
Var OFN32 : tW32OpenFileName;   (* STRUCTURE 32 BITS *)
Begin
  if OS32Bits
     then
       begin
         if @W32GetOpenFileName = Nil
            then
              begin
                if hInstanceComDlg32 = 0
                   then
                     hInstanceComDlg32:=W16LoadLibraryEx32W('ComDlg32',0,0);
                if hInstanceComDlg32 <> 0
                   then
                     @W32GetOpenFileName:=W16GetProcAddress32W(hInstanceComDlg32,'GetOpenFileNameA');
              end;
         if @W32GetOpenFileName <> Nil
            then
              begin
                FillChar(OFN32,SizeOf(OFN32),0);
                OFN32.lStructSize:=SizeOf(tW32OpenFileName);
                OFN32.hWndOwner:=HWND32(OFN.hWndOwner);
                OFN32.hInstance:=OFN.hInstance;
                if OFN.lpstrFilter <> Nil
                   then
                     OFN32.lpstrFilter:=W16GetVDMPointer32W(OFN.lpstrFilter,StrLen(OFN.lpstrFilter));
                if OFN.lpstrCustomFilter <> Nil
                   then
                     OFN32.lpstrCustomFilter:=W16GetVDMPointer32W(OFN.lpstrCustomFilter,StrLen(OFN.lpstrCustomFilter));
                OFN32.nMaxCustFilter:=OFN.nMaxCustFilter;
                OFN32.nFilterIndex:=OFN.nFilterIndex;
                if OFN.lpstrFile <> Nil
                   then
                     OFN32.lpstrFile:=W16GetVDMPointer32W(OFN.lpstrFile,StrLen(OFN.lpstrFile));
                OFN32.nMaxFile:=OFN.nMaxFile;
                if OFN.lpstrFileTitle <> Nil
                   then
                     OFN32.lpstrFileTitle:=W16GetVDMPointer32W(OFN.lpstrFileTitle,StrLen(OFN.lpstrFileTitle));
                OFN32.nMaxFileTitle:=OFN.nMaxFileTitle;
                if OFN.lpstrInitialDir <> Nil
                   then
                     OFN32.lpstrInitialDir:=W16GetVDMPointer32W(OFN.lpstrInitialDir,StrLen(OFN.lpstrInitialDir));
                if OFN.lpstrTitle <> Nil
                   then
                     OFN32.lpstrTitle:=W16GetVDMPointer32W(OFN.lpstrTitle,StrLen(OFN.lpstrTitle));
                OFN32.Flags:=OFN.Flags;
                OFN32.nFileOffset:=OFN.nFileOffset;
                OFN32.nFileExtension:=OFN.nFileExtension;
                if OFN.lpstrDefExt <> Nil
                   then
                     OFN32.lpstrDefExt:=W16GetVDMPointer32W(OFN.lpstrDefExt,StrLen(OFN.lpstrDefExt));
                OFN32.lCustData:=OFN.lCustData;
                Move(OFN.lpfnHook,OFN32.lpfnHook,SizeOf(OFN32.lpfnHook));
                if OFN32.lpfnHook <> Nil
                   then
                     OFN32.lpfnHook:=W16GetVDMPointer32W(OFN32.lpfnHook,vdm_AddressIsReal);
                if OFN.lpTemplateName <> Nil
                   then
                     OFN32.lpTemplateName:=W16GetVDMPointer32W(OFN.lpTemplateName,StrLen(OFN.lpTemplateName));
                FillChar(CallProcParams,SizeOf(CallProcParams),0);
                CallProcParams[0].Valeur:=LongInt(@OFN32);
                CallProcParams[0].Traduire:=True;
                WIN32_GETOPENFILENAME:=APPEL32(1,@W32GetOpenFileName) <> 0;
              end
            else
              WIN32_GETOPENFILENAME:=False;
       end
     else
       WIN32_GETOPENFILENAME:=GetOpenFileName(OFN);
End;

Function WIN32_GETSAVEFILENAME (var OFN : tWIN32OpenFileName) : Boolean;
(* DIALOGUE COMMUN D'ENREGISTREMENT DE FICHIER *)
Var OFN32 : tW32OpenFileName;   (* STRUCTURE 32 BITS *)
Begin
  if OS32Bits
     then
       begin
         if @W32GetSaveFileName = Nil
            then
              begin
                if hInstanceComDlg32 = 0
                   then
                     hInstanceComDlg32:=W16LoadLibraryEx32W('ComDlg32',0,0);
                if hInstanceComDlg32 <> 0
                   then
                     @W32GetSaveFileName:=W16GetProcAddress32W(hInstanceComDlg32,'GetSaveFileNameA');
              end;
         if @W32GetSaveFileName <> Nil
            then
              begin
                FillChar(OFN32,SizeOf(OFN32),0);
                OFN32.lStructSize:=SizeOf(tW32OpenFileName);
                OFN32.hWndOwner:=HWND32(OFN.hWndOwner);
                OFN32.hInstance:=OFN.hInstance;
                if OFN.lpstrFilter <> Nil
                   then
                     OFN32.lpstrFilter:=W16GetVDMPointer32W(OFN.lpstrFilter,StrLen(OFN.lpstrFilter));
                if OFN.lpstrCustomFilter <> Nil
                   then
                     OFN32.lpstrCustomFilter:=W16GetVDMPointer32W(OFN.lpstrCustomFilter,StrLen(OFN.lpstrCustomFilter));
                OFN32.nMaxCustFilter:=OFN.nMaxCustFilter;
                OFN32.nFilterIndex:=OFN.nFilterIndex;
                if OFN.lpstrFile <> Nil
                   then
                     OFN32.lpstrFile:=W16GetVDMPointer32W(OFN.lpstrFile,StrLen(OFN.lpstrFile));
                OFN32.nMaxFile:=OFN.nMaxFile;
                if OFN.lpstrFileTitle <> Nil
                   then
                     OFN32.lpstrFileTitle:=W16GetVDMPointer32W(OFN.lpstrFileTitle,StrLen(OFN.lpstrFileTitle));
                OFN32.nMaxFileTitle:=OFN.nMaxFileTitle;
                if OFN.lpstrInitialDir <> Nil
                   then
                     OFN32.lpstrInitialDir:=W16GetVDMPointer32W(OFN.lpstrInitialDir,StrLen(OFN.lpstrInitialDir));
                if OFN.lpstrTitle <> Nil
                   then
                     OFN32.lpstrTitle:=W16GetVDMPointer32W(OFN.lpstrTitle,StrLen(OFN.lpstrTitle));
                OFN32.Flags:=OFN.Flags;
                OFN32.nFileOffset:=OFN.nFileOffset;
                OFN32.nFileExtension:=OFN.nFileExtension;
                if OFN.lpstrDefExt <> Nil
                   then
                     OFN32.lpstrDefExt:=W16GetVDMPointer32W(OFN.lpstrDefExt,StrLen(OFN.lpstrDefExt));
                OFN32.lCustData:=OFN.lCustData;
                Move(OFN.lpfnHook,OFN32.lpfnHook,SizeOf(OFN32.lpfnHook));
                if OFN32.lpfnHook <> Nil
                   then
                     OFN32.lpfnHook:=W16GetVDMPointer32W(OFN32.lpfnHook,vdm_AddressIsReal);
                if OFN.lpTemplateName <> Nil
                   then
                     OFN32.lpTemplateName:=W16GetVDMPointer32W(OFN.lpTemplateName,StrLen(OFN.lpTemplateName));
                FillChar(CallProcParams,SizeOf(CallProcParams),0);
                CallProcParams[0].Valeur:=LongInt(@OFN32);
                CallProcParams[0].Traduire:=True;
                WIN32_GETSAVEFILENAME:=APPEL32(1,@W32GetSaveFileName) <> 0;
              end
            else
              WIN32_GETSAVEFILENAME:=False;
       end
     else
       WIN32_GETSAVEFILENAME:=GetSaveFileName(OFN);
End;

Function WIN32_CHOOSECOLOR (var CC : tWIN32ChooseColor) : Boolean;
(* DIALOGUE COMMUN DE CHOIX DE COULEUR *)
Var CC32 : tW32ChooseColor;   (* STRUCTURE 32 BITS *)
Begin
  if OS32Bits
     then
       begin
         if @W32ChooseColor = Nil
            then
              begin
                if hInstanceComDlg32 = 0
                   then
                     hInstanceComDlg32:=W16LoadLibraryEx32W('ComDlg32',0,0);
                if hInstanceComDlg32 <> 0
                   then
                     @W32ChooseColor:=W16GetProcAddress32W(hInstanceComDlg32,'ChooseColorA');
              end;
         if @W32ChooseColor <> Nil
            then
              begin
                FillChar(CC32,SizeOf(CC32),0);
                CC32.lStructSize:=SizeOf(tW32ChooseColor);
                CC32.hWndOwner:=HWND32(CC.hWndOwner);
                CC32.hInstance:=CC.hInstance;
                CC32.rgbResult:=CC.rgbResult;
                if CC.lpCustColors <> Nil
                   then
                     CC32.lpCustColors:=W16GetVDMPointer32W(CC.lpCustColors,64);
                CC32.Flags:=CC.Flags;
                CC32.lCustData:=CC.lCustData;
                Move(CC.lpfnHook,CC32.lpfnHook,SizeOf(CC32.lpfnHook));
                if CC32.lpfnHook <> Nil
                   then
                     CC32.lpfnHook:=W16GetVDMPointer32W(CC32.lpfnHook,vdm_AddressIsReal);
                if CC.lpTemplateName <> Nil
                   then
                     CC32.lpTemplateName:=W16GetVDMPointer32W(CC.lpTemplateName,StrLen(CC.lpTemplateName));
                FillChar(CallProcParams,SizeOf(CallProcParams),0);
                CallProcParams[0].Valeur:=LongInt(@CC32);
                CallProcParams[0].Traduire:=True;
                if APPEL32(1,@W32ChooseColor) <> 0
                   then
                     begin
                       CC.rgbResult:=CC32.rgbResult;
                       WIN32_CHOOSECOLOR:=True;
                     end
                   else
                     WIN32_CHOOSECOLOR:=False;
              end
            else
              WIN32_CHOOSECOLOR:=False;
       end
     else
       WIN32_CHOOSECOLOR:=ChooseColor(CC);
End;

Function WIN32_CHOOSEFONT (var CF : tWIN32ChooseFont; var LF : tWIN32LogFont; DC : LongInt) : Boolean;
(* DIALOGUE COMMUN DE CHOIX DE POLICE DE CARACTERES *)
Var CF32 : tW32ChooseFont;   (* STRUCTURE 32 BITS *)
    LF32 : tW32LogFont;      (* ATTRIBUTS 32 BITS DE LA FONTE *)
Begin
  if OS32Bits
     then
       begin
         if @W32ChooseFont = Nil
            then
              begin
                if hInstanceComDlg32 = 0
                   then
                     hInstanceComDlg32:=W16LoadLibraryEx32W('ComDlg32',0,0);
                if hInstanceComDlg32 <> 0
                   then
                     @W32ChooseFont:=W16GetProcAddress32W(hInstanceComDlg32,'ChooseFontA');
              end;
         if @W32ChooseFont <> Nil
            then
              begin
                (* PREPARATION DE LA STRUCTURE DE DEFINITION DES ATTRIBUTS DE LA FONTE *)
                FillChar(LF32,SizeOf(LF32),0);
                LF32.lfHeight:=LF.lfHeight;
                LF32.lfWidth:=LF.lfWidth;
                LF32.lfEscapement:=LF.lfEscapement;
                LF32.lfOrientation:=LF.lfOrientation;
                LF32.lfWeight:=LF.lfWeight;
                LF32.lfItalic:=LF.lfItalic;
                LF32.lfUnderline:=LF.lfUnderline;
                LF32.lfStrikeOut:=LF.lfStrikeOut;
                LF32.lfCharSet:=LF.lfCharSet;
                LF32.lfOutPrecision:=LF.lfOutPrecision;
                LF32.lfClipPrecision:=LF.lfClipPrecision;
                LF32.lfQuality:=LF.lfQuality;
                LF32.lfPitchAndFamily:=LF.lfPitchAndFamily;
                if StrLen(LF.lfFaceName) > 0
                   then
                     StrCopy(LF32.lfFaceName,LF.lfFaceName);
                (* PREPARATION DE LA STRUCTURE D'INTERFACE DU DIALOGUE *)
                FillChar(CF32,SizeOf(CF32),0);
                CF32.lStructSize:=SizeOf(CF32);
                CF32.hWndOwner:=HWND32(CF.hWndOwner);
                CF32.hDC:=DC;
                CF32.lpLogFont:=W16GetVDMPointer32W(@LF32,SizeOf(LF32));
                CF32.iPointSize:=CF.iPointSize;
                CF32.Flags:=CF32.Flags or CF.Flags;
                CF32.rgbColors:=CF.rgbColors;
                CF32.lCustData:=CF.lCustData;
                Move(CF.lpfnHook,CF32.lpfnHook,SizeOf(CF32.lpfnHook));
                if CF32.lpfnHook <> Nil
                   then
                     CF32.lpfnHook:=W16GetVDMPointer32W(CF32.lpfnHook,vdm_AddressIsReal);
                if CF.lpTemplateName <> Nil
                   then
                     CF32.lpTemplateName:=W16GetVDMPointer32W(CF.lpTemplateName,StrLen(CF.lpTemplateName));
                CF32.hInstance:=CF.hInstance;
                if CF.lpszStyle <> Nil
                   then
                     CF32.lpszStyle:=W16GetVDMPointer32W(CF.lpszStyle,lf_FaceSize);
                CF32.nFontType:=CF.nFontType;
                CF32.nSizeMin:=CF.nSizeMin;
                CF32.nSizeMax:=CF.nSizeMax;
                (* APPEL DE LA FONCTION 32 BITS *)
                FillChar(CallProcParams,SizeOf(CallProcParams),0);
                CallProcParams[0].Valeur:=LongInt(@CF32);
                CallProcParams[0].Traduire:=True;
                if APPEL32(1,@W32ChooseFont) <> 0
                   then
                     begin
                       (* RETOUR DE LA STRUCTURE DE DEFINITION DES ATTRIBUTS DE LA FONTE *)
                       LF.lfHeight:=LF32.lfHeight;
                       LF.lfWidth:=LF32.lfWidth;
                       LF.lfEscapement:=LF32.lfEscapement;
                       LF.lfOrientation:=LF32.lfOrientation;
                       LF.lfWeight:=LF32.lfWeight;
                       LF.lfItalic:=LF32.lfItalic;
                       LF.lfUnderline:=LF32.lfUnderline;
                       LF.lfStrikeOut:=LF32.lfStrikeOut;
                       LF.lfCharSet:=LF32.lfCharSet;
                       LF.lfOutPrecision:=LF32.lfOutPrecision;
                       LF.lfClipPrecision:=LF32.lfClipPrecision;
                       LF.lfQuality:=LF32.lfQuality;
                       LF.lfPitchAndFamily:=LF32.lfPitchAndFamily;
                       if StrLen(LF32.lfFaceName) > 0
                          then
                            StrCopy(LF.lfFaceName,LF32.lfFaceName);
                       (* RETOUR DE LA STRUCTURE D'INTERFACE AVEC LE DIALOGUE *)
                       CF.iPointSize:=CF32.iPointSize;
                       CF.rgbColors:=CF32.rgbColors;
                       CF.nFontType:=CF32.nFontType;
                       WIN32_CHOOSEFONT:=True;
                     end
                   else
                     WIN32_CHOOSEFONT:=False;
              end
            else
              WIN32_CHOOSEFONT:=False;
       end
     else
       begin
         CF.lpLogFont:=@LF;
         CF.hDC:=DC;
         WIN32_CHOOSEFONT:=ChooseFont(CF);
       end;
End;

Function WIN32_COMMDLGEXTENDEDERROR : LongInt;
(* RETOURNE LE CODE ETENDU DE LA DERNIERE ERREUR RELATIVE AUX DIALOGUES COMMUNS WINDOWS *)
Begin
  if OS32Bits
     then
       begin
         if @W32CommDlgExtendedError = Nil
            then
              begin
                if hInstanceComDlg32 = 0
                   then
                     hInstanceComDlg32:=W16LoadLibraryEx32W('ComDlg32',0,0);
                if hInstanceComDlg32 <> 0
                   then
                     @W32CommDlgExtendedError:=W16GetProcAddress32W(hInstanceComDlg32,'CommDlgExtendedError');
              end;
         if @W32CommDlgExtendedError <> Nil
            then
              begin
                FillChar(CallProcParams,SizeOf(CallProcParams),0);
                WIN32_COMMDLGEXTENDEDERROR:=APPEL32(0,@W32ChooseColor);
              end
            else
              WIN32_COMMDLGEXTENDEDERROR:=-1;
       end
     else
       WIN32_COMMDLGEXTENDEDERROR:=CommDlgExtendedError;
End;

(* ---------------------------------------------------------------------------------------------------------------------- *)
(*                                                    INITIALISATIONS                                                     *)
(* ---------------------------------------------------------------------------------------------------------------------- *)

BEGIN
  WIN32_INIT;
  ExitProc:=@WIN32_EXIT;
END.
